Package: schmear
Title: Build Structured Data Frame Subtypes
Version: 0.1.0
Authors@R: 
    person("Cory", "McCartan", , "mccartan@psu.edu", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-6251-669X"))
Description: Provides developer-focused helper functions and S3 classes to ease
    the creation of structured subtypes of data frames. Developers can require
    certain columns and types to be present, and can enforce crossing and 
    nesting relationships between values in different columns. Type-specific
    metadata and attributes are preserved through common data frame
    manipulations.
Imports: rlang, cli, vctrs
Suggests: dplyr, pillar, testthat (>= 3.0.0)
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://corymccartan.com/schmear/,
        https://github.com/CoryMcCartan/schmear
BugReports: https://github.com/CoryMcCartan/schmear/issues
Config/testthat/edition: 3
Config/roxygen/version: 8.0.0
Config/roxygen/markdown: TRUE
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-22 23:34:42 UTC; cmccartan
Author: Cory McCartan [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6251-669X>)
Maintainer: Cory McCartan <mccartan@psu.edu>
Repository: CRAN
Date/Publication: 2026-05-29 09:30:11 UTC
Built: R 4.5.2; ; 2026-05-29 10:19:14 UTC; unix
