# Generated by roxygen2: do not edit by hand

S3method("[",sch_df)
S3method("names<-",sch_df)
S3method(dplyr::dplyr_col_modify,sch_df)
S3method(dplyr::dplyr_reconstruct,sch_df)
S3method(dplyr::dplyr_row_slice,sch_df)
S3method(format,sch_schema)
S3method(format,sch_type)
S3method(pillar::tbl_sum,sch_df)
S3method(print,sch_schema)
S3method(print,sch_type)
export(new_sch_df)
export(sch_any)
export(sch_character)
export(sch_coerce)
export(sch_custom)
export(sch_date)
export(sch_datetime)
export(sch_factor)
export(sch_inherits)
export(sch_integer)
export(sch_list_of)
export(sch_logical)
export(sch_multiple)
export(sch_nest)
export(sch_numeric)
export(sch_others)
export(sch_schema)
export(sch_validate)
export(validate_sch_df)
