
<!-- README.md is generated from README.Rmd. Please edit that file -->

# schmear <a href="https://corymccartan.com/schmear/"><img src="man/figures/logo.png" align="right" height="138" alt="schmear logo" /></a>

<!-- badges: start -->

[![Codecov test
coverage](https://codecov.io/gh/CoryMcCartan/schmear/graph/badge.svg)](https://app.codecov.io/gh/CoryMcCartan/schmear)
[![R-CMD-check](https://github.com/CoryMcCartan/schmear/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/CoryMcCartan/schmear/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

Provides developer-focused helper functions and S3 classes to ease the
creation of structured subtypes of data frames which require certain
columns and types to be present, and which preserve special attributes
through common data frame manipulations.

## Installation

You can install the released version of schmear from CRAN with:

``` r
install.packages("schmear")
```

You can install the development version from
[GitHub](https://github.com/CoryMcCartan/schmear) with:

``` r
# install.packages("pak")
pak::pak("CoryMcCartan/schmear")
```
